<?php

namespace Drupal\imce_rename_plugin\Plugin\ImcePlugin;

use Drupal\imce\Imce;
use Drupal\imce\ImcePluginBase;
use Drupal\imce\ImceFM;

/**
 * Defines Imce Rename plugin.
 *
 * @ImcePlugin(
 *   id = "rename",
 *   label = "Rename",
 *   operations = {
 *     "rename" = "opRename"
 *   }
 * )
 */
class Rename extends ImcePluginBase {

  /**
   * {@inheritdoc}
   */
  public function permissionInfo() {
    return [
      'rename_images' => $this->t('Rename images'),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildPage(array &$page, ImceFM $fm) {
    if ($fm->hasPermission('rename_images')) {
      $page['#attached']['library'][] = 'imce_rename_plugin/drupal.imce.rename';
    }
  }

  /**
   * Operation handler: rename.
   */
  public function opRename(ImceFM $fm) {
    $old_name = $fm->selection[0]->name;
    $folder = $fm->activeFolder;
    $old_file = $folder->getItem($old_name);

    // Check type.
    if ($old_file->type == 'file') {
      // Crop string up to 50 characters.
      $new_name = mb_substr($fm->getPost('new_name'), 0, 50);
      // Transliteration name.
      $new_name = \Drupal::service('transliteration')->transliterate($new_name);
      // Replace space to dash.
      $new_name = str_replace(' ', '-', $new_name);;
      // Delete special symbols.
      $new_name = preg_replace('/[^\w_-]+/u', '', $new_name);
      // Add extension to file name.
      $new_name = $new_name . '.' . pathinfo($old_name, PATHINFO_EXTENSION);
      $new_uri = $folder->getUri() . '/' . $new_name;
      $old_uri = $folder->getUri() . '/' . $old_file->name;

      if (file_exists($new_uri)) {
        drupal_set_message($this->t('Failed to rename file "@old_file" to "@new_file" because "@new_file" already exists', [
          '@old_file' => utf8_encode(basename($old_uri)),
          '@new_file' => utf8_encode(basename($new_uri)),
        ]), 'error');
        return;
      }

      // Check access to write file and try to change chmod.
      if (!is_writable($old_uri) && !chmod($old_uri, 0664)) {
        drupal_set_message($this->t('No permissions to write file "@old_file". Please upload the file via IMCE.', [
          '@old_file' => utf8_encode(basename($old_uri)),
        ]), 'error');
        return;
      }

      $file = Imce::getFileEntity($old_uri);
      // Create entity when there is no entity for the file.
      $file = empty($file) ? Imce::createFileEntity($old_uri) : $file;
      $move = file_move($file, $new_uri, FILE_EXISTS_ERROR);
      $move->setFilename($new_name);
      $move->save();

      // Validate message.
      if ($move) {
        drupal_set_message($this->t('Rename successful! Renamed "@old_file" to "@new_file"', [
          '@old_file' => utf8_encode(basename($old_uri)),
          '@new_file' => utf8_encode(basename($new_uri)),
        ]));
        $folder->addFile($new_name)->addToJs();
        $folder->getItem($old_name)->removeFromJs();
      }
      else {
        drupal_set_message($this->t('Failed to rename file "@old_file" to "@new_file".', [
          '@old_file' => utf8_encode(basename($old_uri)),
          '@new_file' => utf8_encode(basename($new_uri)),
        ]), 'error');
      }
    }
    elseif ($old_file->type == 'folder') {
      drupal_set_message($this->t('Sorry, but rename a folder is not available yet'));
    }
  }

}
